use super::{run_chroot, Image};
use crate::setup::{network::WifiSecurity, Setup};
use anyhow::Context as _;
use std::{
	fs::{self, File},
	io::Write as _
};

pub fn init_network(img: &Image, setup: &Setup) -> anyhow::Result<()> {
	eprintln!("Preparing network ...");

	let mut interfaces = File::create(img.root.0.join("etc").join("network").join("interfaces"))?;
	for (name, ty) in [("lo", "loopback"), ("eth0", "dhcp")] {
		writeln!(interfaces, "auto {name}")?;
		writeln!(interfaces, "iface {name} inet {ty}")?;
		writeln!(interfaces)?;
	}
	writeln!(interfaces, "hostname {}", setup.os.host.hostname)?;
	fs::write(
		img.root.0.join("etc").join("resolv.conf"),
		r#"nameserver 1.0.0.1
nameserver 1.1.1.1
nameserver 2606:4700:4700::1001
nameserver 2606:4700:4700::1111
		"#
	)?;
	run_chroot(img, "root", ["rc-update", "add", "networking", "default"])?;

	if setup.network.tailscale {
		run_chroot(img, "root", "apk add tailscale; rc-update add tailscale default")?;
		fs::write(
			img.root.0.join("usr").join("local").join("bin").join("tailscale-up"),
			r#"#!/bin/sh
for i in 0 1 2 3 4; do
	ping -c 2 -W 1 1.1.1.1 && break
	if [ "$i" == "4" ]; then exit 1; fi
	echo "Waiting for network ..."
	sleep 2s
done

set -e

echo "Connecting to tailscale ..."
timeout -v 2m tailscale up
"#
		)?;
		fs::write(
			img.root.0.join("etc").join("init.d").join("tailscale-up"),
			r#"#!/sbin/openrc-run
command="/usr/local/bin/tailscale-up"
command_background=false
depend() {
	after ntpd
	need tailscale
}"#
		)?;
		run_chroot(
			img,
			"root",
			"chmod +x /usr/local/bin/tailscale-up /etc/init.d/tailscale-up; rc-update add tailscale-up default"
		)?;
	}

	if !setup.network.wifi.is_empty() {
		run_chroot(img, "root", "echo 'brcmfmac' >> /etc/modules")?;
		run_chroot(img, "root", ["apk", "add", "iw", "iwd", "wireless-tools", "wireless-regdb"])?;
		run_chroot(img, "root", ["rc-update", "add", "iwd", "default"])?;

		let iwd = img.root.0.join("etc").join("iwd");
		fs::create_dir_all(&iwd).context("Unable to create /etc/iwd directory")?;
		fs::write(
			iwd.join("main.conf"),
			r#"# main iwd configuration file

[General]
EnableNetworkConfiguration=true

[Network]
EnableIPv6=true
NameResolvingService=none
"#
		)
		.context("Unable to write /etc/iwd/main.conf")?;

		let iwd = img.root.0.join("var").join("lib").join("iwd");
		fs::create_dir_all(&iwd).context("Unable to create /var/lib/iwd directory")?;
		for wifi in &setup.network.wifi {
			let ssid = &wifi.ssid;
			match &wifi.security {
				WifiSecurity::None => {
					fs::write(iwd.join(format!("{ssid}.open")), "\n")?;
				},
				WifiSecurity::WpaPsk { password } => {
					fs::write(iwd.join(format!("{ssid}.psk")), format!("[Security]\nPassphrase={password}\n"))?;
				},
				WifiSecurity::WpaEap { identity, password } => {
					let mut file = File::create(iwd.join(format!("{ssid}.8021x")))?;
					writeln!(file, "[Security]")?;
					writeln!(file, "EAP-Method=PWD")?;
					writeln!(file, "EAP-Identity={identity}")?;
					writeln!(file, "EAP-Password={password}")?;
					writeln!(file)?;
					writeln!(file, "[Settings]")?;
					writeln!(file, "AutoConnect=true")?;
				}
			}
		}
	}

	Ok(())
}
